#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  #
  # Enrich with location
  #
  module SensorBi
    # Id of the sensor
    BI = 13

    def self.included(base)
      base.extend(ClassMethods)
      base.send(:include, InstanceMethods)

      base.class_eval do
        # Increase available list of sensors
        self.available_sensors += [BI]

        # Import values for BI
        #self.import_columns_for[BI] = [:ip, :license_id].freeze
        self.import_columns_for[BI] = [:license_id].freeze

        # -------------------------------------------------------------
        # Validations
        # -------------------------------------------------------------
        #validates :ip, presence: true,
                    #   uniqueness: { scope: :real_parent_id },
        #               if: proc { type == BI }

        #validate :uniqueness_ip, if: proc { type == BI }

        DittoCode::Exec.if 'ENTERPRISE' do
        validates_presence_of :license_id, if: proc { type == BI }
        validate :empty_license_buckets_for_bi, if: proc { type == BI }
        end

        # -------------------------------------------------------------
        # JSON serialization
        # -------------------------------------------------------------

        alias_method_chain :as_json, :bi

        # -------------------------------------------------------------
        # Before and after methods
        # -------------------------------------------------------------
        #before_update :update_cache_bi,
        #              if: proc { type == BI && ip_changed? }
        after_create :create_cache_bi, if: proc { type == BI }
        after_create :create_chef_node_bi,
                     if: proc { type == BI }
        #after_update :update_ip_bi, if: proc { type == BI }
        after_save :data_to_client_proxy_bi,
                   if: proc { type == BI && real_parent_id }
        before_destroy :delete_data_from_client_proxy_bi,
                       if: proc { type == BI && real_parent_id }
      end
    end

    module ClassMethods
      define_method("to_partial_path_#{BI}") do
        'sensors/bi_sensor'
      end
    end

    module InstanceMethods

      def is_bi?
        type == BI
      end

      # Return the descendant bi sensors of the domain
      def get_bi
        self.all_children(true).where(type: Sensor::BI)
      end

      # Return last time when sensor was checked
      define_method("last_check_in_#{BI}") do
        Time.at(check_in_time) if check_in_time
      end

      define_method("icon_#{BI}") do
        'icons/bi.png'
      end

      define_method("full_name_#{BI}") do
        name
      end

      define_method("secondary_name_#{BI}") do
        name
      end

      define_method("complete_name_#{BI}") do
        name
      end

      define_method("type_to_s_#{BI}") do
        'bi'
      end

      define_method("type_humanize_#{BI}") do
        'BI'
      end

      define_method("chef_node_#{BI}") do
        Chef::Node.load("rbbi-#{id}") rescue nil
      end

      define_method("chef_client_#{BI}") do
        Chef::ApiClient.load("rbbi-#{id}")
      end

      private

      def uniqueness_ip
        sensors = Sensor.where(type: type, ip: ip, real_parent_id: real_parent_id)
        sensors = sensors.where.not(id: id) if id
        if sensors.any?
          errors.add(:ip, "#{I18n.t 'lib_bi.unique'}")
        end
      end

      # Check if license can be associated to current sensor
      #
      def empty_license_buckets_for_bi
        return unless license
        return if license_id_was == license_id
        errors.add(:license_id, "#{I18n.t 'lib_bi.full'}") if license.in_use_bi >= license.limit_bi
      end

      def create_cache_bi
        Rails.cache.write "rb-webui:sensor:#{type}:#{name}", ip
      end

      def update_cache_bi
        Rails.cache.write "rb-webui:sensor:#{type}:#{name}", ip
      end

      def create_chef_node_bi
        node_json = {
          'name' => "rbbi-#{id}",
          'run_list' => ['role[bi-sensor]', "role[#{chef_name}]"],
          'attributes' => {
            'rbname' => name,
            'redBorder' => {
              'sensor_uuid' => uuid,
              'monitors' => [],
              'parent_id' => real_parent_id
            }
          }
        }

        node = Chef::Node.from_hash(node_json)
        node.create
      end

      def update_ip_bi
        role = chef_role
        node = chef_node
        role.save
        node.save
      end

      #
      # Return the sensor as json with variables from BI
      #
      def as_json_with_bi(options = {})
        result = as_json_without_bi(options)
        #result.merge!({ ip: ip }.as_json) if self.is_bi?
        result
      end

      # Method used to complete values about ancestors on chef node
      define_method("update_chef_domain_values_#{BI}") do
        update_chef_domain_values_in_node
      end

      # Create the chef role for BI sensors
      #
      define_method "create_chef_role_#{BI}" do
        run_list = ['role[bi-sensor]', "role[#{parent.chef_name}]"]
        bi_attributes = {
          'redBorder' => {
            'rsyslog' => {},
            'proxy' => {},
            'snmp' => { 'trap_server' => [] },
            'monitors' => []
          }
        }
        builder = Redborder::Chef::Role::Builder.new(
                    self, run_list, override_attributes: bi_attributes
                  )
        builder.create
      end

      define_method "destroy_chef_node_#{BI}" do
        chef_node.destroy
      end

      # Store the current data of the Chef node in the client_proxy parent
      #
      def data_to_client_proxy_bi
        # Get the role of the parent
        role = real_parent.chef_role

        # Update attributes
        role.override_attributes['redborder']['sensors_mapping'] ||= {}
        role.override_attributes['redborder']['sensors_mapping']['bi'] ||= {}
        role.override_attributes['redborder']['sensors_mapping']['bi'][name] =
          chef_node['redborder']
        # Save it
        role.save
      end

      def delete_data_from_client_proxy_bi
        client_proxy = real_parent
        role = client_proxy.chef_role

        role.override_attributes['redborder']['sensors_mapping']['bi'].delete(name)
        role.save
      end
    end
  end
end

Sensor.send(:include, RedBorder::SensorBi)
