#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

require 'syslog'

module Redborder
  # This class notificate an alarm to the users using several methods. Now,
  # available methods are:
  #   - Syslog
  #   - GCM push
  #   - Email
  #
  class AlarmNotificator
    attr_reader :alarm, :result, :type

    # Receive the alarm to notify and the result of query
    #
    # @param alarm [Alarm] Alarm to notify
    # @param result [Hash] Result of the query based on alarm conditions
    #
    def initialize(alarm, result)
      @alarm = alarm
      @result = result
      # We can define an alarm with an upper, a bottom or both limits. In the case we
      # define two limits, the alarm will be fired only by one of those limits.
      # That the reason I set here the "type" of the alarm.
      @type = @result[:where].to_s
    end

    # Send a notification based on the options from alarm
    #
    def notify
      notify_email if @alarm.notify_to_email
      notify_syslog if @alarm.notify_to_syslog
      notify_slack if @alarm.notify_to_slack
    end

    # Send a notificaiton using the email
    #
    def notify_email
      AlarmMailer.alarm(alarm, result).deliver_now
    end
    
    # Send a notificaiton using syslog
    #
    def notify_syslog
      ::Syslog.open('redBorder') do |s|
        s.alert(
          "#{I18n.t 'lib_redborder.alarm'} %s #{I18n.t 'lib_redborder.fired'} #{@type} #{I18n.t 'lib_redborder.limit'} %s %s #{I18n.t 'lib_redborder.with'} %s %s #{I18n.t 'lib_redborder.in'} %s <-> %s",
          alarm.name, limit.to_s, units, value, units, result[:start_time].to_s,
          result[:end_time].to_s
        )
      end
    end

    def notify_slack
      require "uri"
      require "json"
      require "openssl"
      require "net/http"
      if @alarm
        begin
          uri = URI(@alarm.slack_webhook)
          http = Net::HTTP.new(uri.host, uri.port)
          http.use_ssl = true
          http.verify_mode = OpenSSL::SSL::VERIFY_PEER
          request = Net::HTTP::Post.new(uri.request_uri)
          request.add_field('Content-Type', 'application/json')
          request.body = { text:"[#{Setting.company}] Alarm #{@alarm.name} #{@alarm.product_type} Severity #{@alarm.severity.name} fired! Domain Alarm Triggered: #{Setting.webpage_url} " }.to_json
          response = http.request(request)

        rescue Exception => e
          raise "#{Time.now} [ERROR] Error on Slack request: #{e}"
        end
      end
    end

    private

    # Get the value of the result
    #
    def value
      @value ||= (result[:value] / (10.0**alarm.send("multiplier_#{@type}"))).round(2)
    end

    # Return the alarm unit
    #
    def units
      @units ||= alarm.units("multiplier_#{@type}".to_sym)
    end

    # Return the limit of the alarm
    #
    def limit
      @limit ||= alarm.send("#{@type}_limit")
    end

    # Return the severity color of the alarm
    #
    def severity_colour
      @alarm.severity.try(:bg_color) || '#777777'
    end

    # Build the message of the alarm
    #
    def mobile_message
      # Base message
      message = {
        name: @alarm.name,
        start_time: @result[:start_time].to_i,
        end_time: @result[:end_time].to_i,
        type: @type,
        value: value,
        server_key: Setting.server_key,
        severity_colour: severity_colour
      }

      # Merge limits
      message.merge(
        "#{@type}_limit".to_sym => limit,
        "#{@type}_units".to_sym => units
      )
    end
  end
end
